@@ -30,6 +30,7 @@ typedef struct _conv_func {
 conv_func_t msgconv_func_list[] = {
     { "alternative", convert_alternative },
     { "multipart", convert_multipart },
+    { "extension", convert_extension },
     { "none", convert_none },
 };
 #define CONVFUNC_LIST_SIZE (sizeof(msgconv_func_list) / sizeof(conv_func_t))
@@ -73,6 +74,26 @@ void free_harmlessconf(command_list_t *list)
 }
 
 
+/**
+ * free_mimetypes
+ *     MIMEタイプを定義した構造体のメモリを解放する
+ * args:
+ *     mimetype_list_t *list  MIMEタイプを定義した構造体
+ */
+void free_mimetypes(mimetype_list_t *list)
+{
+    mimetype_list_t *tmp, *del;
+
+    if (list != NULL) {
+        for (tmp = list; tmp != NULL; tmp = tmp->next, free(del)) {
+            free(tmp->extension);
+            free(tmp->mimetype);
+            del = tmp;
+        }
+    }
+}
+
+
 char *read_harmlessconf(char *file, void **pointer)
 {
     static char errbuf[1024] = "";
@@ -99,7 +120,8 @@ char *read_harmlessconf(char *file, void **pointer)
     }
 
     for (nline = 1; fgets(line, MAX_CONFIG_LINE + 1, fp) != NULL; nline++) {
-        command_list_t current_list = {NULL, NULL, NULL, NULL, -1, NULL, NULL, NULL};
+        command_list_t current_list = {NULL, NULL, NULL, NULL, 
+                                       -1, 0, 0, NULL, 0, NULL};
 
         tail = strchr(line, '\n');
         if (tail == NULL) {
@@ -141,6 +163,13 @@ char *read_harmlessconf(char *file, void **pointer)
         }
         fth = save;
 
+        // 先頭の@の有無のチェック
+        if (*fth == '@') {
+            // メッセージ抑制フラグを立て、次のポインタへ
+            current_list.msg_restraint = MSG_RESTRAIN;
+            fth++;
+        }
+
         // 指定されたコマンドのチェック
         if (*fth != '/') {
             current_list.command_type = FUNCTION;
@@ -149,6 +178,10 @@ char *read_harmlessconf(char *file, void **pointer)
                 goto error;
             }
 
+            if (strcmp(current_list.command_name, "extension") == 0) {
+                current_list.msg_restraint = MSG_RESTRAIN;
+            }
+
             // 内部関数をセット
             for (i = 0; i < CONVFUNC_LIST_SIZE; i++) {
                 if (strcasecmp(msgconv_func_list[i].item, fth)) {
@@ -231,6 +264,126 @@ end:
     return errbuf[0] == '\0' ? NULL : errbuf;
 }
 
+
+/**
+ * read_mimetypes
+ *     MIMEタイプ定義ファイルを読み込み、
+ *     拡張子をキーとしたリスト構造体へ格納する
+ * Args:
+ *     char *filepath  MIMEタイプ定義ファイルのファイルパス
+ *     void **pointer  情報が格納されたリスト構造体へのポインタ
+ * Return:
+ *     errbuf          エラーメッセージ
+ */
+char *read_mimetypes(char *filepath, void **pointer) {
+    static char errbuf[1024] = "";
+    char *errmsg;
+    char *tail, *save, *tmp_ex, *save_ex;
+    char line[MAX_CONFIG_LINE + 1];
+    char *mime, *extensions;
+    FILE *fp = NULL;
+    int nline;
+    mimetype_list_t *making_list = NULL, *new_list = NULL, *list_end = NULL;
+
+    /* ファイルを読み込めるか確認して開く */
+    errmsg = is_readable_file(filepath);
+    if (errmsg) {
+        sprintf(errbuf, MIME_ERR_READ, filepath);
+        free_mimetypes(making_list);
+        return errbuf[0] == '\0' ? NULL : errbuf;
+    }
+
+    fp = fopen(filepath, "r");
+    if (fp == NULL) {
+        sprintf(errbuf, MIME_ERR_OPEN, filepath, strerror(errno));
+        free_mimetypes(making_list);
+        return errbuf[0] == '\0' ? NULL : errbuf;
+    }
+
+    /* 1行ずつ読み込む */
+    for (nline = 1; fgets(line, MAX_CONFIG_LINE + 1, fp); nline++) {
+        /* 一時的に情報を格納する構造体を初期化する 
+           {extension, mimetype, next} */
+        mimetype_list_t current_list = {NULL, NULL, NULL};
+
+        /* 改行を探す */
+        tail = strchr(line, '\n');
+        if (tail == NULL) {
+            sprintf(errbuf, MIME_TOOLONGLINE, filepath, nline);
+            fclose(fp);
+            free_mimetypes(making_list);
+            return errbuf[0] == '\0' ? NULL : errbuf;
+        }
+        /* 行末を終端にする */
+        *tail = '\0';
+
+        /* コメントと空行を読み飛ばす */
+        if ((line[0] == '#') || (line[0] == '\0')) {
+            continue;
+        }
+
+        /* MIMEタイプ部と拡張子部の2つに分割 */
+        mime = strtok_r(line, " \t", &save);
+        if (mime == NULL) {
+            /* 空行なので読み飛ばす */
+            continue;
+        }
+
+        /* タブと空白を飛ばし、余った部分を拡張子部として確保 */
+        while (isblank(*save)) save++;
+        extensions = save;
+
+       /* 拡張子がない場合は読み飛ばす */
+        if (save == tail) {
+            continue;
+        }
+
+        /* 拡張子を分割し構造体へ格納する */
+        while ((tmp_ex = strtok_r(extensions, " \t", &save_ex)) != NULL) {
+            if ((current_list.extension = strdup(tmp_ex)) == NULL) {
+                sprintf(errbuf, MIME_ERR_MEM);
+                fclose(fp);
+                free_mimetypes(making_list);
+                return errbuf[0] == '\0' ? NULL : errbuf;
+            }
+            if ((current_list.mimetype = strdup(mime)) == NULL) {
+                sprintf(errbuf, MIME_ERR_MEM);
+                fclose(fp);
+                free_mimetypes(making_list);
+                return errbuf[0] == '\0' ? NULL : errbuf;
+            }
+
+            /* 情報が入った構造体を別の構造体へ */
+            new_list = calloc(1, sizeof(mimetype_list_t));
+            if (new_list == NULL) {
+                sprintf(errbuf, MIME_ERR_MEM);
+                fclose(fp);
+                free_mimetypes(making_list);
+                return errbuf[0] == '\0' ? NULL : errbuf;
+            }
+            *new_list = current_list;
+
+            /* 構造体をリスト構造にする */
+            if (making_list == NULL) {
+                making_list = new_list;
+                list_end = new_list;
+            } else {
+                list_end->next = new_list;
+                list_end = new_list;
+            }
+            extensions = NULL;
+        }
+    }
+    *pointer = making_list;
+
+    if (fp != NULL) {
+        fclose(fp);
+    }
+
+    return errbuf[0] == '\0' ? NULL : errbuf;
+}
+
+
 #define MIME_BEFORE 0
 #define MIME_AFTER 1
 #define FILE_BEFORE 2
@@ -249,9 +402,15 @@ int msg_convert(GMimeObject *part, GMimeObject **new, harmless_proc_arg_t *arg)
     command_list_t *c_p;
     command_list_t *command_list = arg->cfg->cf_harmlessconf;
 
-    GMimeContentType *top_content_type_object = g_mime_object_get_content_type(part);
+    // convert_alternativeを通過していたら、取得したMIMEタイプを使用
+    // 通過していない場合、メール本文から取得
+    if (arg->mime_extension != NULL) {
+        content_type = arg->mime_extension;
+    } else {
+        GMimeContentType *top_content_type_object = g_mime_object_get_content_type(part);
+        content_type = g_mime_content_type_to_string(top_content_type_object);
+    }
 
-    content_type = g_mime_content_type_to_string(top_content_type_object);
     if (content_type == NULL) {
         log("Content-Type is missing.");
         return HARMLESS_NG;
@@ -262,7 +421,6 @@ int msg_convert(GMimeObject *part, GMimeObject **new, harmless_proc_arg_t *arg)
 
     // mimeタイプ判定のループ
     for (c_p = command_list; c_p != NULL; c_p = c_p->next) {
-
         // 元のMIMEタイプと受信メールのMIMEタイプが一致したら0でif文に入らない
         // 元のMIMEタイプに*が指定されていた場合はif文に入らない
         // 違っていたら入る
@@ -275,11 +433,14 @@ int msg_convert(GMimeObject *part, GMimeObject **new, harmless_proc_arg_t *arg)
             case FUNCTION:
                 _ret = (c_p->in_func)(part, new, arg);
                 if (_ret == HARMLESS_NG) {
-                    output_conv_log(CONV_ERR_EXEC_IN, c_p->command_name, arg->maildata);
+                    if (arg->mime_extension != NULL) {
+                        output_conv_log(CONV_ERR_EXEC_IN, c_p->command_name, arg->maildata);
+                    }
                     free(content_type);
                     return HARMLESS_NG;
+                } else if (_ret == NO_MATCH_EXTENSION) {
+                    continue;
                 }
-
                 break;
             case COMMAND:
                 _ret = exec_command_with_gmime(part, new, c_p->arg_list, arg);
@@ -324,13 +485,19 @@ int msg_convert(GMimeObject *part, GMimeObject **new, harmless_proc_arg_t *arg)
 
             taglist[MIME_AFTER].st_str = c_p->mime_after;
 
-            ret = strcat_proc_message(
-                &(arg->message),
-                &(arg->message_length),
-                arg->cfg->cf_harmlessmessageconvert,
-                taglist,
-                4
-            );
+            // 渡された構造体のメッセージ抑制フラグを確認
+            if (c_p->msg_restraint == MSG_NO_RESTRAIN) {
+                ret = strcat_proc_message(
+                    &(arg->message),
+                    &(arg->message_length),
+                    arg->cfg->cf_harmlessmessageconvert,
+                    taglist,
+                    4
+                );
+            } else {
+                // メッセージ抑制の場合、retにOKの値を入れる
+                ret = HARMLESS_OK;
+            }
 
             free(content_type);
             if (newfilename != NULL) {
@@ -357,8 +524,10 @@ int convert_alternative(GMimeObject *part, GMimeObject **new, harmless_proc_arg_
     GMimeMultipart *top;
 
     command_list_t *cp;
+    command_list_t *cp2;
     command_list_t *clist = arg->cfg->cf_harmlessconf;
 
+
     if (!GMIME_IS_MULTIPART(part)) {
         return HARMLESS_NG;
     }
@@ -445,6 +614,82 @@ int convert_none(GMimeObject *part, GMimeObject **new, harmless_proc_arg_t *arg)
     return HARMLESS_OK;
 }
 
+/**
+ * convert_extension (add 20170126)
+ *     arg->cfg->cf_mimetypesと添付ファイルの拡張子を比較し、
+ *     extensionコマンドを除き再びmsg_convert関数へ渡す
+ */
+int convert_extension(GMimeObject *part, GMimeObject **new, harmless_proc_arg_t *arg)
+{
+    mimetype_list_t *m_p, *mime_list = arg->cfg->cf_mimetypes;
+    char *attachment_extension;
+    const char *filename;
+    int res, ret = 0;
+
+    /* argのMaxExtensionDepthの検査 */
+    if (arg->used_extension == USED_EXTENSION) {
+        log(ERR_REPEAT_EXTENSION, arg->mime_extension);
+        return HARMLESS_NG;
+    }
+
+    /* 添付ファイル名を取得 */
+    filename = estimate_filename(part);
+
+    /* 添付ファイル名の有無を判断 */
+    if (filename != "") {
+        /* 拡張子を取得 */
+        attachment_extension = strrchr(filename, '.');
+
+        if (attachment_extension == NULL || 
+            strcmp(attachment_extension, ".") == 0) {
+            /* 拡張子が無ければ次のharmless.confの行の処理へ  */
+            log(NOT_FOUND_EXTENSION, filename);
+            return NO_MATCH_EXTENSION;
+        }
+
+        /* ドット以降を取得 */
+        attachment_extension++;
+
+    } else {
+        /* ファイル名が無ければ次のharmless.confの行の処理へ */
+        log(NOT_FOUND_FILENAME);
+        return NO_MATCH_EXTENSION;
+    }
+
+    /* 拡張子を検索する */
+    for (m_p = mime_list; m_p != NULL; m_p = m_p->next) {
+        res = strcmp(m_p->extension, attachment_extension);
+        if (res == 0) {
+            /* 拡張子が一致したら定義されたMIMEタイプを取得 */
+            arg->mime_extension = strdup(m_p->mimetype);
+            if (arg->mime_extension == NULL) {
+                log(EXTE_ERR_MEM);
+                return HARMLESS_NG;
+            }
+            break;
+        }
+    }
+
+    /* 拡張子の一致がなければ次のharmless.confの行の処理へ */
+    if (arg->mime_extension == NULL) {
+        log(NO_EXTENSION_INMIME, filename);
+        return NO_MATCH_EXTENSION;
+    }
+
+    /* argにextensionコマンドを通ったフラグを立てる */
+    arg->used_extension = USED_EXTENSION;
+
+    /* msg_convertに変更を加えた構造体を渡す */
+    ret = msg_convert(part, new, arg);
+
+    /* 構造体を渡してしまったのでMIMEタイプとフラグを元に戻す */
+    arg->mime_extension = NULL;
+    arg->used_extension = 0;
+
+    /* 再処理した結果を返す */
+    return ret;
+}
+
 /**
  * すごい適当なので、だれかうまいこと直して下さい
  * （exec_command.cに移行すべきです）
